"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const auth = __importStar(require("../auth"));
const server_1 = require("../__mocks__/server");
const msw_1 = require("msw");
const querystring_1 = __importDefault(require("querystring"));
// import { mockDate } from '../__mocks__/mockDate';
const auth_1 = require("../auth");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const dayjs_1 = __importDefault(require("dayjs"));
const handlers_1 = require("../__mocks__/handlers");
const testData = {
    ADMIN_USER: 'test@test.pl',
    ADMIN_PASS: 'hello',
    TENANT: 'tenant123',
    VENUE: 'venue1',
    BASE_URL: 'http://localhost',
    BASIC_AUTH: 'abc',
};
test('getTokenWithPassword', () => __awaiter(void 0, void 0, void 0, function* () {
    const authSuccess = yield auth.authorizeWithUserPass(testData.BASE_URL, testData.TENANT, testData.BASIC_AUTH, testData.ADMIN_USER, testData.ADMIN_PASS);
    expect(authSuccess).toBe(true);
}));
test('getTokenWithRefreshToken', () => __awaiter(void 0, void 0, void 0, function* () {
    const authSuccess = yield auth.authorizeWithRefreshToken(testData.BASE_URL, testData.TENANT, testData.BASIC_AUTH, handlers_1.goodAuthResponseBody.refresh_token);
    expect(authSuccess).toBe(true);
}));
test('getAuthData - makes only 1 API call with multiple concurrent request for access_token having refresh_token', () => __awaiter(void 0, void 0, void 0, function* () {
    server_1.server.use(msw_1.rest.post(authUrl, (req, res, ctx) => {
        return res(ctx.status(200), ctx.json(handlers_1.goodAuthResponseBody));
    }));
    const dispatchRequest = jest.fn();
    server_1.server.events.on('request:start', dispatchRequest);
    const tokenProvider = {
        getRefreshToken: jest
            .fn()
            .mockResolvedValue(handlers_1.goodAuthResponseBody.refresh_token),
        setRefreshToken: jest.fn().mockResolvedValue(undefined),
        clearRefreshToken: jest.fn().mockResolvedValue(undefined),
    };
    // Clear _authData from other tests
    auth.clearAuthData('any_tenant', tokenProvider);
    const results = yield Promise.all(Array(10)
        .fill(0)
        .map(() => auth.authDataProvider(testData, tokenProvider)));
    expect(results.every((result) => result.token === handlers_1.goodAuthResponseBody.access_token)).toBe(true);
    expect(dispatchRequest).toHaveBeenCalledTimes(1);
}));
test('getAuthData - makes 10 API calls with multiple concurrent request if network error', () => __awaiter(void 0, void 0, void 0, function* () {
    server_1.server.use(msw_1.rest.post(authUrl, (req, resp, ctx) => {
        return resp.networkError('Failed to connect');
    }));
    const dispatchRequest = jest.fn();
    server_1.server.events.on('request:start', dispatchRequest);
    const tokenProvider = {
        getRefreshToken: jest
            .fn()
            .mockResolvedValue(handlers_1.goodAuthResponseBody.refresh_token),
        setRefreshToken: jest.fn().mockResolvedValue(undefined),
        clearRefreshToken: jest.fn().mockResolvedValue(undefined),
    };
    // Clear _authData from other tests
    auth.clearAuthData('any_tenant', tokenProvider);
    const results = yield Promise.all(Array(10)
        .fill(0)
        .map(() => auth.authDataProvider(testData, tokenProvider)));
    expect(results.every((result) => result.token === '')).toBe(true);
    expect(dispatchRequest).toHaveBeenCalledTimes(10);
}));
test('getAuthData - makes max 1 API call with same refresh_token if response is 4XX', () => __awaiter(void 0, void 0, void 0, function* () {
    server_1.server.use(msw_1.rest.post(authUrl, (req, res, ctx) => {
        return res(ctx.status(400), ctx.json({ error: 'Invalid grant' }));
    }));
    const dispatchRequest = jest.fn();
    server_1.server.events.on('request:start', dispatchRequest);
    const tokenProvider = {
        getRefreshToken: jest
            .fn()
            .mockResolvedValue(handlers_1.goodAuthResponseBody.refresh_token),
        setRefreshToken: jest.fn().mockResolvedValue(undefined),
        clearRefreshToken: jest.fn().mockResolvedValue(undefined),
    };
    // Clear _authData from other tests
    auth.clearAuthData('any_tenant', tokenProvider);
    const results = yield Promise.all(Array(10)
        .fill(0)
        .map(() => auth.authDataProvider(testData, tokenProvider)));
    expect(results.every((result) => result.token === '')).toBe(true);
    expect(dispatchRequest).toHaveBeenCalledTimes(1);
}));
test('getTokenWithRefreshToken - wrong token', () => __awaiter(void 0, void 0, void 0, function* () {
    const authSuccess = yield auth.authorizeWithRefreshToken(testData.BASE_URL, testData.TENANT, testData.BASIC_AUTH, 'invalid_token');
    expect(authSuccess).toBe(false);
}));
const authUrl = `${testData.BASE_URL}/auth-oauth2/oauth/token`;
test('getAuthData - return existing token if it is still valid ', () => __awaiter(void 0, void 0, void 0, function* () {
    const validToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().add(20, 'second').unix(),
    }, 'secret');
    const refreshHandler = {
        getRefreshToken: (tenant) => new Promise((resolve) => resolve('TEST_REFRESH_TOKEN')),
        setRefreshToken: (tenant) => new Promise((resolve) => resolve()),
        clearRefreshToken: (tenant) => new Promise((resolve) => resolve()),
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: true,
    };
    auth.setAuthData('tenant1', {
        expires_in: '3600',
        access_token: validToken,
        UUID: 'user1',
        refresh_token: 'refresh1',
    }, refreshHandler);
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe(validToken);
}));
test('getAuthData - clears refresh token if JWT expired and does NOT call oauth2 api', () => __awaiter(void 0, void 0, void 0, function* () {
    const expiredToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().subtract(1, 'second').unix(),
    }, 'secret');
    const clearRefreshToken = jest.fn();
    const dispatchRequest = jest.fn();
    server_1.server.events.on('request:start', dispatchRequest);
    const refreshHandler = {
        getRefreshToken: (tenant) => expiredToken,
        setRefreshToken: (tenant) => { },
        clearRefreshToken: clearRefreshToken,
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: false,
    };
    auth.setAuthData(testData.TENANT, {
        expires_in: '3600',
        access_token: expiredToken,
        UUID: 'user1',
        refresh_token: expiredToken,
    }, refreshHandler);
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe('');
    expect(UUID).toBe('');
    expect(clearRefreshToken).toHaveBeenCalledTimes(1);
    expect(clearRefreshToken).toHaveBeenCalledWith(testData.TENANT);
    expect(dispatchRequest).toHaveBeenCalledTimes(0);
}));
test('getAuthData - does call oauth2 api clears refresh token if refresh_token is not a JWT token', () => __awaiter(void 0, void 0, void 0, function* () {
    const nonJwtToken = 'non-jwt-token';
    const clearRefreshToken = jest.fn();
    const dispatchRequest = jest.fn();
    server_1.server.events.on('request:start', dispatchRequest);
    const refreshHandler = {
        getRefreshToken: (tenant) => nonJwtToken,
        setRefreshToken: (tenant) => { },
        clearRefreshToken: clearRefreshToken,
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: false,
    };
    auth.setAuthData(testData.TENANT, {
        expires_in: '3600',
        access_token: nonJwtToken,
        UUID: 'user1',
        refresh_token: nonJwtToken,
    }, refreshHandler);
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe('');
    expect(UUID).toBe('');
    expect(clearRefreshToken).toHaveBeenCalledTimes(1);
    expect(clearRefreshToken).toHaveBeenCalledWith(testData.TENANT);
    expect(dispatchRequest).toHaveBeenCalledTimes(1);
}));
test('getAuthData - use refresh token to retrieve new token if existing token expired', () => __awaiter(void 0, void 0, void 0, function* () {
    const expiredToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().subtract(1, 'second').unix(),
    }, 'secret');
    const validRefreshToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().add(24, 'hour').unix(),
    }, 'secret');
    const validAccessToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().add(1, 'hour').unix(),
    }, 'secret2');
    server_1.server.use(msw_1.rest.post(authUrl, (req, resp, ctx) => {
        const params = querystring_1.default.parse('' + req.body);
        const grantType = params['grant_type'];
        if (grantType === 'refresh_token') {
            const refreshToken = params['refresh_token'];
            if (refreshToken === validRefreshToken) {
                return resp(ctx.status(200), ctx.json({
                    access_token: validAccessToken,
                    expires_in: '3600',
                    UUID: 'user_uuid123',
                    refresh_token: validRefreshToken,
                }));
            }
        }
        return resp(ctx.status(403));
    }));
    const refreshHandler = {
        getRefreshToken: (tenant) => validRefreshToken,
        setRefreshToken: (tenant) => { },
        clearRefreshToken: (tenant) => { },
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: false,
    };
    auth.setAuthData('tenant1', {
        expires_in: '3600',
        access_token: expiredToken,
        UUID: 'user1',
        refresh_token: 'refresh1',
    }, refreshHandler);
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe(validAccessToken);
    expect(UUID).toBe('user_uuid123');
}));
test('getAuthData - use refresh token to retrieve new token if existing token is to expire within 10 seconds', () => __awaiter(void 0, void 0, void 0, function* () {
    const expiredToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().subtract(1, 'second').unix(),
    }, 'secret');
    const validRefreshToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().add(24, 'hour').unix(),
    }, 'secret');
    const validAccessToken = jsonwebtoken_1.default.sign({
        exp: (0, dayjs_1.default)().add(1, 'hour').unix(),
    }, 'secret2');
    server_1.server.use(msw_1.rest.post(authUrl, (req, resp, ctx) => {
        const params = querystring_1.default.parse('' + req.body);
        const grantType = params['grant_type'];
        if (grantType === 'refresh_token') {
            const refreshToken = params['refresh_token'];
            if (refreshToken === validRefreshToken) {
                return resp(ctx.status(200), ctx.json({
                    access_token: validAccessToken,
                    expires_in: '3600',
                    UUID: 'user_uuid123',
                    refresh_token: validRefreshToken,
                }));
            }
        }
        return resp(ctx.status(403));
    }));
    const refreshHandler = {
        getRefreshToken: (tenant) => validRefreshToken,
        setRefreshToken: (tenant) => { },
        clearRefreshToken: (tenant) => { },
    };
    const ctx = {
        BASE_URL: testData.BASE_URL,
        TENANT: testData.TENANT,
        BASIC_AUTH: testData.BASIC_AUTH,
        anonymousAuth: true,
    };
    auth.setAuthData('tenant1', {
        expires_in: '3600',
        access_token: expiredToken,
        UUID: 'user1',
        refresh_token: 'refresh1',
    }, refreshHandler);
    const { token, UUID } = yield auth.authDataProvider(ctx, refreshHandler, false);
    expect(token).toBe(validAccessToken);
    expect(UUID).toBe('user_uuid123');
}));
const userDataUrl = `${testData.BASE_URL}/auth-api/api/me`;
const validUserData = {
    firstName: 'testname',
    email: 'email@email.com',
    UUID: 'uuid_1',
};
beforeEach(() => {
    // Setup server
    server_1.server.use(msw_1.rest.get(userDataUrl, (req, res, ctx) => {
        if (req.headers.get('authorization') ===
            `Bearer ${handlers_1.goodAuthResponseBody.access_token}`) {
            return res(ctx.status(200), ctx.json(validUserData));
        }
        return res(ctx.status(401));
    }));
    server_1.server.use(msw_1.rest.post(userDataUrl, (req, res, ctx) => {
        if (req.headers.get('authorization') ===
            `Bearer ${handlers_1.goodAuthResponseBody.access_token}`) {
            return res(ctx.status(200), ctx.json(true));
        }
        return res(ctx.status(401));
    }));
});
test('getLoggedUserData', () => __awaiter(void 0, void 0, void 0, function* () {
    const userData = yield (0, auth_1.getLoggedUserData)(testData.BASE_URL, handlers_1.goodAuthResponseBody.access_token);
    expect(userData).toMatchObject(validUserData);
}));
test('updateUserData', () => __awaiter(void 0, void 0, void 0, function* () {
    const result = yield (0, auth_1.updateUserData)(testData.BASE_URL, handlers_1.goodAuthResponseBody.access_token, {
        phone: '123',
        firstName: 'testname',
    });
    expect(result).toBe(true);
}));
test('updateUserData - invalid token', () => __awaiter(void 0, void 0, void 0, function* () {
    const onError = jest.fn();
    yield (0, auth_1.updateUserData)(testData.BASE_URL, 'INVALID_TEST_TOKEN', {
        phone: '123',
        firstName: 'testname',
    }).catch(onError);
    expect(onError).toHaveBeenCalled();
}));
describe('token expiry function', () => {
    test('returns expired after expiry & 10s before', () => {
        expect((0, auth_1.isTokenExpired)(jsonwebtoken_1.default.sign({
            exp: (0, dayjs_1.default)().subtract(10, 'second').unix(),
        }, 'secret'))).toBe(true);
        expect((0, auth_1.isTokenExpired)(jsonwebtoken_1.default.sign({
            exp: (0, dayjs_1.default)().add(10, 'second').unix(),
        }, 'secret'))).toBe(true);
    });
    test('returns not expired 11s before expiry', () => {
        expect((0, auth_1.isTokenExpired)(jsonwebtoken_1.default.sign({
            exp: (0, dayjs_1.default)().add(11, 'second').unix(),
        }, 'secret'))).toBe(false);
    });
    test('return expired for non JWT token without throwing', () => {
        expect((0, auth_1.isTokenExpired)('non-jwt-access-token')).toBe(true);
        expect(() => (0, auth_1.isTokenExpired)('non-jwt-access-token')).not.toThrowError();
    });
});
